package gov.llnl.babel.symbols;

import java.util.HashMap;
import java.util.Set;
import java.util.HashSet;
import java.util.List;
import java.util.Map;
import java.util.ArrayList;
import java.util.Iterator;

/**
 * The <code>Struct</code> contains a list of named types.
 * 
 * @author kumfert
 */
public class Struct extends Symbol {
  /**
   * Holds a list of {@link Item}'s.
   */
  private List d_items;

  public static class Item {
    private String d_name;
    private Type   d_type;

    public Item(String name, Type dtype)
    {
      d_name = name;
      d_type = dtype;
    }

    public String getName() { return d_name; }
    public Type   getType() { return d_type; }
  };

  public Struct(SymbolID id, Comment comment) {
    super(id, Symbol.STRUCT, comment);
    d_items = new ArrayList();
  }

  public Struct(SymbolID id, Comment comment, Metadata metadata) {
    super(id, Symbol.STRUCT, comment, metadata);
    d_items = new ArrayList();
  }

  public Set getSymbolReferences() {
    HashSet s = new HashSet();
    Iterator i = d_items.iterator();
    while (i.hasNext()) {
      Type t = ((Item)i.next()).getType();
      SymbolID id = t.getSymbolID();
      if (id != null) {
        s.add(id);
      }
    }
    return s;
  }

  public Set getAllSymbolReferences() {
    Set result = getSymbolReferences();
    result.add(getSymbolID());
    return result;
  }

  public Set getBasicArrayRefs() {
    return null;
  }
  
  /**
   * 
   * @return ordered list of items. Each element is an {@link Item}.
   */
  public List getItems() { 
    return d_items;
  }

  public int addItem(Item i)
  {
    d_items.add(i);
    return d_items.size();
  }

  /**
   * Insert a new type and name into the  struct
   * @return new number of items in the struct
   */
  public int addItem( String name, Type type ) { 
    d_items.add(new Item(name, type));
    return d_items.size();
  }
}
