//
// File:        AddDefaultsVisitor.java
// Package:     
// Copyright:   (c) 2006 The Regents of the University of California
// Release:     $Name$
// Revision:    @(#) $Revision: 5295 $
// Date:        $Date: 2006-03-24 14:02:42 -0800 (Fri, 24 Mar 2006) $
// Description: 
// 
package gov.llnl.babel.visitor;

import gov.llnl.babel.BabelConfiguration;
import gov.llnl.babel.msg.MsgList;
import gov.llnl.babel.msg.UserMsg;
import gov.llnl.babel.msg.UserMsg2;
import gov.llnl.babel.ast.ASTNode;
import gov.llnl.babel.ast.SIDLFile;
import gov.llnl.babel.ast.ImportClause;
import gov.llnl.babel.ast.RequireClause;
import gov.llnl.babel.ast.ClassType;
import gov.llnl.babel.ast.InterfaceType;
import gov.llnl.babel.ast.Package;
import gov.llnl.babel.ast.ScopedID;
import gov.llnl.babel.ast.Method;
import gov.llnl.babel.ast.Argument;
import gov.llnl.babel.ast.FixedType;
import gov.llnl.babel.ast.ArrayType;
import gov.llnl.babel.ast.RArrayType;
import gov.llnl.babel.ast.Type;
import gov.llnl.babel.ast.Extents;
import gov.llnl.babel.ast.Name;
import gov.llnl.babel.ast.ImplementsList;
import gov.llnl.babel.ast.ExtendsList;


import java.util.ArrayList;
import java.util.ListIterator;
import java.util.Iterator;
import java.util.HashMap;
import java.util.Vector;
import java.util.LinkedList;
/**
 * Iterates over a AST a second time to decorate a primed SymbolTable
 * with all the details about classes, interfaces, and structs
 * 
 * Type resolution is largely performed in this stage.
 * 
 * @see gov.llnl.babel.visitor.SymbolTablePrimer
 * @author kumfert
 */
public class AddDefaultsVisitor  extends Visitor {
  
  private final static String SIDL_CLASS
    = "." + BabelConfiguration.getBaseClass();
  
  private final static String SIDL_INTERFACE 
    = "." + BabelConfiguration.getBaseInterface();
  
  private final static String SIDL_IMPLICIT_THROW 
    = "." + BabelConfiguration.getRuntimeException();

  protected MsgList d_msgs;

  public AddDefaultsVisitor(MsgList msgs) {
    super();
    d_msgs = msgs;
  }
  
  /** Add defaults at the Class Level.  
   *  1. If the class is SIDL_BASECLASS, implement SIDL_BASEINTERFACE
   *  2. If the class doesn't inherit anything, extend SIDL_BASECLASS
   */
  public Object visitClassType(ClassType node, Object data) { 
    if (node.getFQN().equals(SIDL_CLASS)) {
      if(node.getImplementsList().size() == 0) { 
        ImplementsList implementsList = node.getImplementsList();
        implementsList.add(new ScopedID(SIDL_INTERFACE));
      }
    } else if ( node.getExtends() == null ) {
      node.setExtends(new ScopedID(SIDL_CLASS));
    }

    Object result = super.visitClassType(node,data); //Visit children
    return result;
  }
  
  /** Add defaults as the interface level
   *  1. If it's not SIDL_BASEINTERFACE, and it has no extends, extend
   *     SIDL_BASEINTERFACE
   */
  public Object visitInterfaceType(InterfaceType node, Object data) { 
    if (!node.getFQN().equals(SIDL_INTERFACE) && node.getExtends().size() == 0) {
      node.getExtends().add(new ScopedID(SIDL_INTERFACE));
    }
    
    //Visit children
    Object result = super.visitInterfaceType(node,data);
    return result;    
  }


  /** Method level defaults
   *  1. Add implicit Runtime Exception to every throws list
   */
  public Object visitMethod(Method node, Object data) { 
    ListIterator i_thrws = node.getThrowsList().listiterator();
    boolean has_implicit_ex = false;
    for(; i_thrws.hasNext();) {
      ScopedID id = (ScopedID) i_thrws.next();
      if(id.toString().equals(SIDL_IMPLICIT_THROW)) {
        has_implicit_ex = true;
      }
    }
    if(!has_implicit_ex) {
      i_thrws.add(new ScopedID(SIDL_IMPLICIT_THROW));
    }
    Object result = super.visitMethod(node,data);  //Visit children
    return result;
  }
}
